//+------------------------------------------------------------------+
//|                                          Guppy MMA oscilator.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_level1  0
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern string TimeFrame    = "current time frame";
extern int    AppliedPrice = 0;
extern double Speed        = 0.0;
extern bool   Adaptive     = true;
extern int    SignalPeriod = 13;
extern bool   Interpolate  = true;

//
//
//
//
//

extern bool  alertsOn                = true;
extern bool  alertsOnCurrent         = false;
extern bool  alertsOnLinesCross      = true;
extern bool  alertsOnGmmaZeroCross   = false;
extern bool  alertsOnSignalZeroCross = false;
extern bool  alertsMessage           = true;
extern bool  alertsSound             = false;
extern bool  alertsEmail             = false;

//
//
//
//
//

double buffer1[];
double buffer2[];
double trend1[];
double trend2[];
double trend3[];
double periods[]={3,5,8,10,12,15,30,35,40,45,50,60};
int    persize;

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   CalculatingValue;
bool   ReturningBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,buffer1);
   SetIndexBuffer(1,buffer2);
   SetIndexBuffer(2,trend1);
   SetIndexBuffer(3,trend2);
   SetIndexBuffer(4,trend3);
   persize =ArraySize(periods);
      
      //
      //
      //
      //
      //
         
         IndicatorFileName = WindowExpertName();
         CalculatingValue  = (TimeFrame=="calculateGMMA"); if (CalculatingValue) return(0);
         ReturningBars     = (TimeFrame=="returnBars");    if (ReturningBars)    return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //

   IndicatorShortName("Guppy OMA "+timeFrameToString(timeFrame));
   return(0);
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   double alpha = 2.0/(1.0+SignalPeriod);
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ReturningBars)  { buffer1[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (CalculatingValue || timeFrame == Period())
   {
      for(i=limit; i>=0; i--)
      {
         double sum  = 0;
         double price = iMA(NULL,0,1,0,MODE_SMA,AppliedPrice,i);
         for(int j=0; j<persize; j++)
         {
            if (periods[j]<30)
                 sum += iAverage(price,periods[j],Speed,Adaptive,i,j*7);
            else sum -= iAverage(price,periods[j],Speed,Adaptive,i,j*7);
         }
         buffer1[i] = sum*10.0;
         buffer2[i] = buffer2[i+1]+alpha*(buffer1[i]-buffer2[i+1]);
         trend1[i]  = trend1[i+1];
         trend2[i]  = trend2[i+1];
         trend3[i]  = trend3[i+1];
      
         //
         //
         //
         //
         //
         
         if (buffer1[i] > buffer2[i]) trend1[i] =  1;
         if (buffer1[i] < buffer2[i]) trend1[i] = -1;
         if (buffer1[i] > 0)          trend2[i] =  1;
         if (buffer1[i] < 0)          trend2[i] = -1;
         if (buffer2[i] > 0)          trend3[i] =  1;
         if (buffer2[i] < 0)          trend3[i] = -1;
      }
      if (!CalculatingValue) manageAlerts();
      return(0);         
   }

   //
   //
   //
   //
   //
      
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         buffer1[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateGMMA",AppliedPrice,Speed,Adaptive,SignalPeriod,0,y);
         buffer2[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateGMMA",AppliedPrice,Speed,Adaptive,SignalPeriod,1,y);
         trend1[i]  = trend1[i+1];
         trend2[i]  = trend2[i+1];
         trend3[i]  = trend3[i+1];

         //
         //
         //
         //
         //
         
            if (buffer1[i] > buffer2[i]) trend1[i] =  1;
            if (buffer1[i] < buffer2[i]) trend1[i] = -1;
            if (buffer1[i] > 0)          trend2[i] =  1;
            if (buffer1[i] < 0)          trend2[i] = -1;
            if (buffer2[i] > 0)          trend3[i] =  1;
            if (buffer2[i] < 0)          trend3[i] = -1;

            if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
            {
               buffer1[i+k] = k*factor*buffer1[i+n] + (1.0-k*factor)*buffer1[i];
               buffer2[i+k] = k*factor*buffer2[i+n] + (1.0-k*factor)*buffer2[i];
            }
   }
   
   //
   //
   //
   //
   //
   
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+  
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
         
         //
         //
         //
         //
         //
         
         if (alertsOnLinesCross)
         {
            static datetime alert1Time    = 0;
            static string   alert1Message = "";
               if (trend1[whichBar] != trend1[whichBar+1])
               if (trend1[whichBar] == 1)
                     doAlert(whichBar,alert1Time,alert1Message,"GMMA crossed signal line up");
               else  doAlert(whichBar,alert1Time,alert1Message,"GMMA crossed signal line down");
         }            
         if (alertsOnGmmaZeroCross)
         {
            static datetime alert2Time    = 0;
            static string   alert2Message = "";
              if (trend2[whichBar] != trend2[whichBar+1])
              if (trend2[whichBar] == 1)
                     doAlert(whichBar,alert2Time,alert2Message,"GMMA crossed zero line up");
              else   doAlert(whichBar,alert2Time,alert2Message,"GMMA crossed zero line down");
         }            
         if (alertsOnSignalZeroCross)
         {
            static datetime alert3Time    = 0;
            static string   alert3Message = "";
              if (trend3[whichBar] != trend3[whichBar+1])
              if (trend3[whichBar] == 1)
                     doAlert(whichBar,alert3Time,alert3Message,"GMMA signal crossed zero line up");
              else   doAlert(whichBar,alert3Time,alert3Message,"GMMA signal crossed zero line down");
         }            
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+  
//
//
//
//
//

void doAlert(int forBar, datetime& previousTime, string& previousAlert, string doWhat)
{
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," GMMA OMA - ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"GMMA OMA"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define E1  0
#define E2  1
#define E3  2
#define E4  3
#define E5  4
#define E6  5
#define res 6

double stored[][84];

//
//
//
//
//

double iAverage(double price, double averagePeriod, double const, bool adaptive,int i, int ashift=0)
{
   int r = Bars-i-1;
   
   if (ArrayRange(stored,0) != Bars) { ArrayResize(stored,Bars); }
   
   double e1=stored[r-1][E1+ashift];  double e2=stored[r-1][E2+ashift];
   double e3=stored[r-1][E3+ashift];  double e4=stored[r-1][E4+ashift];
   double e5=stored[r-1][E5+ashift];  double e6=stored[r-1][E6+ashift];

   //
   //
   //
   //
   //

      if (adaptive && (averagePeriod > 1))
      {
         double minPeriod = averagePeriod/2.0;
         double maxPeriod = minPeriod*5.0;
         int    endPeriod = MathCeil(maxPeriod);
         double signal    = MathAbs((price-stored[r-endPeriod][res+ashift]));
         double noise     = 0.00000000001;

            for(int k=1; k<endPeriod; k++) noise=noise+MathAbs(price-stored[r-k][res+ashift]);

         averagePeriod = ((signal/noise)*(maxPeriod-minPeriod))+minPeriod;
      }
      
      //
      //
      //
      //
      //
      
      double alpha = (2.0+const)/(1.0+const+averagePeriod);

      e1 = e1 + alpha*(price-e1); e2 = e2 + alpha*(e1-e2); double v1 = 1.5 * e1 - 0.5 * e2;
      e3 = e3 + alpha*(v1   -e3); e4 = e4 + alpha*(e3-e4); double v2 = 1.5 * e3 - 0.5 * e4;
      e5 = e5 + alpha*(v2   -e5); e6 = e6 + alpha*(e5-e6); double v3 = 1.5 * e5 - 0.5 * e6;

   //
   //
   //
   //
   //

   stored[r][E1+ashift]  = e1;  stored[r][E2+ashift] = e2;
   stored[r][E3+ashift]  = e3;  stored[r][E4+ashift] = e4;
   stored[r][E5+ashift]  = e5;  stored[r][E6+ashift] = e6;
   stored[r][res+ashift] = price;
   return(v3);
}